#!/bin/sh

# ViaThinkSoft Personal WebBase
# Publisher Shell Script
# (C) 2009 ViaThinkSoft

# Konstanten

SCRIPT_NAME=publish.sh;
BASE=/daten/webbase;
DEV_NAME=dev;
DEVELOPER_SYSTEM=$BASE/$DEV_NAME;
WORKING_DIR=$BASE/private;
TEMP_NAME=publish_temp;
NAME_IN_ARCHIVE=webbase;
PRELOAD_DIR=/daten/dateiserver/preload;
SNAPSHOT_DIR=/daten/webbase/downloads;
TIMESTAMP_FILE=build_timestamp.txt;
DOWNLOAD_TIMESTAMP_FILE=webbase_daily_snapshop_timestamp.txt;
SNAPSHOT_NAME=webbase_daily_snapshot.zip;
PUBLISH_NAME=webbase_publish.zip;

# Prfen, ob Argumente OK

if [ "$1" != "publish" -a "$1" != "snapshot" ]
then
	echo "Usage: $SCRIPT_NAME publish|snapshot [-silent]"
	exit 1
fi

if [ "$2" != "" -a "$2" != "-silent" ]
then
	echo "Usage: $SCRIPT_NAME publish|snapshot [-silent]"
	exit 1
fi

if [ "$3" != "" ]
then
	echo "Usage: $SCRIPT_NAME publish|snapshot [-silent]"
	exit 1
fi

# Start des Codes

if [ "$2" != "-silent" ]
then
	echo "Vorbereitung...";
fi

# Arbeitsverzeichnis wechseln
cd $WORKING_DIR/

# Das Temp-Verzeichnis lschen
if [ -d $WORKING_DIR/$TEMP_NAME/ ]
then
	rm -R $WORKING_DIR/$TEMP_NAME/
fi

# Oder eine Konfliktdatei lschen
if [ -f $WORKING_DIR/$TEMP_NAME ]
then
	rm $WORKING_DIR/$TEMP_NAME
fi

# Das aktuelle Developer-System kopieren und umbenennen
mkdir $WORKING_DIR/$TEMP_NAME/
cp -R $DEVELOPER_SYSTEM/ $WORKING_DIR/$TEMP_NAME/
mv $WORKING_DIR/$TEMP_NAME/$DEV_NAME $WORKING_DIR/$TEMP_NAME/$NAME_IN_ARCHIVE

# Die Datumsangaben des Systems, der Module und Designs
# anhand der jngsten Datei aktualisieren
/usr/bin/php5 $WORKING_DIR/$TEMP_NAME/$NAME_IN_ARCHIVE/private/date_upd.php

# Dieses Publish-Script dazupacken
cp $SCRIPT_NAME $WORKING_DIR/$TEMP_NAME/$NAME_IN_ARCHIVE/private/

# Sowie einen Zeitstempels der Verffentlichung
date > $WORKING_DIR/$TEMP_NAME/$NAME_IN_ARCHIVE/$TIMESTAMP_FILE

# Genau diese Zeit nochmal als PHP-Timestamp fr die Webseite
if [ "$1" == "snapshot" ]
then
	date +"%s" > $WORKING_DIR/$TEMP_NAME/$DOWNLOAD_TIMESTAMP_FILE
fi

# Alle Thumbs.db von Windows lschen
find $WORKING_DIR/$TEMP_NAME/$NAME_IN_ARCHIVE/ -name 'Thumbs.db' -exec rm {} \;

# Alle Textdateien auf CRLF normieren
# Alternativ: -nolist
if [ "$2" != "-silent" ]
then
	echo "DOSCONV sucht nach Dateien, die nicht auf CRLF-Basis normiert sind:"
	/usr/bin/php5 /daten/scripts/tools/dosconv.php $WORKING_DIR/$TEMP_NAME/$NAME_IN_ARCHIVE/ -fix
else
	/usr/bin/php5 /daten/scripts/tools/dosconv.php $WORKING_DIR/$TEMP_NAME/$NAME_IN_ARCHIVE/ -fix > /dev/nul
fi

# Die Standard Konfigurationsdaten wieder herstellen
rm $WORKING_DIR/$TEMP_NAME/$NAME_IN_ARCHIVE/includes/config.inc.php
cp $WORKING_DIR/$TEMP_NAME/$NAME_IN_ARCHIVE/includes/config.inc.default.php $WORKING_DIR/$TEMP_NAME/$NAME_IN_ARCHIVE/includes/config.inc.php

# Temporre Verzeichnisse in den Ausgangszustand zurcksetzen

# Popper's tmp/ darf komplett geleert werden
POPPER_TEMP=modules/user_popper/system/tmp;
rm -R $WORKING_DIR/$TEMP_NAME/$NAME_IN_ARCHIVE/$POPPER_TEMP/*

# Net2FTP hat im Ausgangszustand 3 Dateien im temp/
NET2FTP_TEMP_COPY=net2ftp_temp;
NET2FTP_TEMP=modules/user_net2ftp/system/temp;
mv $WORKING_DIR/$TEMP_NAME/$NAME_IN_ARCHIVE/$NET2FTP_TEMP $WORKING_DIR/$TEMP_NAME/$NET2FTP_TEMP_COPY
mkdir $WORKING_DIR/$TEMP_NAME/$NAME_IN_ARCHIVE/$NET2FTP_TEMP
mv $WORKING_DIR/$TEMP_NAME/$NET2FTP_TEMP_COPY/chmod_this_dir_to_777.txt $WORKING_DIR/$TEMP_NAME/$NAME_IN_ARCHIVE/$NET2FTP_TEMP/chmod_this_dir_to_777.txt
mv $WORKING_DIR/$TEMP_NAME/$NET2FTP_TEMP_COPY/.htaccess $WORKING_DIR/$TEMP_NAME/$NAME_IN_ARCHIVE/$NET2FTP_TEMP/.htaccess
mv $WORKING_DIR/$TEMP_NAME/$NET2FTP_TEMP_COPY/index.php $WORKING_DIR/$TEMP_NAME/$NAME_IN_ARCHIVE/$NET2FTP_TEMP/index.php
rm -R $WORKING_DIR/$TEMP_NAME/$NET2FTP_TEMP_COPY

# compatibility/index.ph_ umbenennen (wegen WildEdit-Ersetzungen nicht .php genannt)
COMPAT_NAME=compatibility;
if [ -f $WORKING_DIR/$TEMP_NAME/$NAME_IN_ARCHIVE/$COMPAT_NAME/index.ph_ ]
then
	mv $WORKING_DIR/$TEMP_NAME/$NAME_IN_ARCHIVE/$COMPAT_NAME/index.ph_ $WORKING_DIR/$TEMP_NAME/$NAME_IN_ARCHIVE/$COMPAT_NAME/index.php
fi

if [ "$2" != "-silent" ]
then
	echo "Erzeuge ein neues Personal WebBase Package...";
fi

# ZIP-Namen festlegen

if [ "$1" == "snapshot" ]
then
	ZIP_NAME=$SNAPSHOT_NAME;
fi

if [ "$1" == "publish" ]
then
	ZIP_NAME=$PUBLISH_NAME;
fi

# Erst alte ZIP Datei lschen
if [ -f $TEMP_NAME/$ZIP_NAME ]
then
	rm $TEMP_NAME/$ZIP_NAME
fi

# Dann zippen (Verzeichnis relativ)
cd $TEMP_NAME
zip -r -q $ZIP_NAME $NAME_IN_ARCHIVE/
cd ..

if [ "$1" == "snapshot" ]
then
	# Snap-Shot zur Verfgung stellen
	mv $TEMP_NAME/$ZIP_NAME $SNAPSHOT_DIR/
	chown homepage:homepage $SNAPSHOT_DIR/$ZIP_NAME
	mv $WORKING_DIR/$TEMP_NAME/$DOWNLOAD_TIMESTAMP_FILE $SNAPSHOT_DIR/$DOWNLOAD_TIMESTAMP_FILE
	chown homepage:homepage $SNAPSHOT_DIR/$DOWNLOAD_TIMESTAMP_FILE
fi

if [ "$1" == "publish" ]
then
	# Oder in den Preload-Ordner verschieben
	mv $TEMP_NAME/$ZIP_NAME $PRELOAD_DIR/
	chown preload:ftpuser $PRELOAD_DIR/$ZIP_NAME
fi

# Im Falle, dass etwas schiefgelaufen ist... die ZIP, die verschoben werden sollte, lschen
if [ -f $TEMP_NAME/$ZIP_NAME ]
then
	rm $TEMP_NAME/$ZIP_NAME
fi

# Und das Temp-Verzeichnis lschen
rm -R $WORKING_DIR/$TEMP_NAME/

if [ "$2" != "-silent" ]
then
	if [ "$1" == "snapshot" ]
	then
		echo "Das erzeuge Personal WebBase Package ist nun als Snap-Shot zur Verfgung gestellt worden.";
	fi
	
	if [ "$1" == "publish" ]
	then
		echo "Das erzeuge Personal WebBase Package ist nun in den Preload-Ordner verschoben worden.";
	fi
fi

exit 0
